/**********************************************************
 *      LCD DRIVER for 82C55 Digital I/O                  *
 *      SW4057                                            *
 *      REV 1.00                                          *
 *      5/24/99                                           *
 *                                                        *
 *      (c)1999 Micro/sys, Inc.                           *
 *      Glendale, CA                                      *
 *                                                        *
 *      This file contains the routines used to access    *
 *      an LCD display interfaced to an 82C55.            *
 **********************************************************/
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <time.h>
#include "LCD8255.h"

extern int DIOBASE;

/************************************************
 *      delay_CONTROL()                         *
 *                                              *
 *      This function will create a delay in    *
 *      55ms timer ticks                        *
 ************************************************/
void delay_CONTROL(unsigned int tick_delay)
{
	unsigned char tick_wait, tick_done;
	unsigned char far* biostick;

	biostick =(unsigned char far*)0x0040006C;

	tick_done = tick_delay + *biostick;


	while( tick_done != *biostick )
		;
}

/**************************************************
 *      wait_for_LCD()                            *
 *                                                *
 *      This function will wait until the LCD     *
 *      is finished with its current operation.   *
 **************************************************/
void wait_for_LCD(void)
{
unsigned char status;     

	do
	{
		status = read_control_LCD();
	}
	while (status & 128);               /* CHECK THE LCD BUSY BIT */
}

/************************************************
 *      read_control_LCD()                      *
 *                                              *
 *      This function will read the control     *
 *      (instruction) register of the LCD.      *
 ************************************************/
unsigned char read_control_LCD(void)
{
        unsigned char ret_LCD, tempch;

        /* CONFIGURE PORTA OF 8255 AS INPUTS (PORTB OUT, PORTC OUT) */
        outp(DIOBASE + DIOCMD, PORTAIN);

        /* SEND COMMAND TO READ FROM THE LCD (R/W ENABLED) */
        outp(DIOBASE + DIOPORTB, LCDRD);
	
        /* ENABLE R/W AND E SIGNALS TO LCD */
        outp(DIOBASE + DIOPORTB, LCDRD | LCD_E);
	
	/* READ FROM THE LCD */
        tempch = inp(DIOBASE + DIOPORTA);

        /* THE FOLLOWING LINE REVERSES THE DATA BITS TO SIMPLIFY */
        /* THE INTERFACE CABLE - SEE DOCUMENTATION FOR DETAILS */
        ret_LCD = reverse_data(tempch);  

	/* DISABLE E SIGNAL */
        outp(DIOBASE + DIOPORTB, LCDRD );
	
	return(ret_LCD);
}
							
/*****************************************************
 *      write_control_LCD()                          *
 *                                                   *
 *      This function will write a byte to the       *
 *      control (instruction) register of the LCD.   *
 *****************************************************/
void write_control_LCD(unsigned char ch)
{
        /* CONFIGURE PORTA OF 8255 AS OUTPUTS (PORTB OUT, PORTC OUT) */
        outp(DIOBASE + DIOCMD, PORTAOUT);

        /* THE FOLLOWING LINE REVERSES THE DATA BITS TO SIMPLIFY */
        /* THE INTERFACE CABLE - SEE DOCUMENTATION FOR DETAILS */
        ch = reverse_data(ch);   

        /* CLOCK ch INTO 8255 PORTA FOR LCD */
        outp(DIOBASE + DIOPORTA, ch);


	/* ENABLE THE E SIGNAL TO LCD */
        outp(DIOBASE + DIOPORTB, LCD_E);

	/* DISABLE THE E SIGNAL TO LCD */
        outp(DIOBASE + DIOPORTB, 0 );
}

