/**********************************************************
 *      LCD DRIVER for 82C55 Digital I/O                  *
 *      SW4057                                            *
 *      REV 1.00                                          *
 *      5/24/99                                           *
 *                                                        *
 *      (c)1999 Micro/sys, Inc.                           *
 *      Glendale, CA                                      *
 *                                                        *
 *      This file contains the routines used to access    *
 *      an LCD display interfaced to an 82C55.            *
 **********************************************************/

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <time.h>
#include "LCD8255.h"

extern int DIOBASE;

/**************************************************
 *      write_data_LCD()                          *
 *                                                *
 *      This function will write a byte to the    *
 *      data register of the LCD device.          * 
 **************************************************/
void write_data_LCD(unsigned char ch)
{
        /* CONFIGURE PORTA (DATA) OF 8255 AS OUTPUTS */
        outp(DIOBASE + DIOCMD, PORTAOUT);

        /* ENABLE REGSEL SIGNAL TO LCD */
        outp(DIOBASE + DIOPORTB, LCDDATA);

        /* THE FOLLOWING LINE REVERSES THE DATA BITS TO SIMPLIFY */
        /* THE INTERFACE CABLE - SEE DOCUMENTATION FOR DETAILS */
        ch = reverse_data(ch);  

        /* CLOCK ch INTO 8255 PORTA FOR LCD */
        outp(DIOBASE + DIOPORTA, ch);

	/* ENABLE E SIGNAL TO LCD */
        outp(DIOBASE + DIOPORTB, LCDDATA | LCD_E);

	/* DISABLE E SIGNAL TO LCD */
        outp(DIOBASE + DIOPORTB, LCDDATA );
}

/**************************************************
 *      read_data_LCD()                           *
 *                                                *
 *      This function will read a byte from       *
 *      the data register of the LCD device.      *
 **************************************************/
unsigned char read_data_LCD(void)
{
        unsigned char ret_LCD, tempch;

        /* CONFIGURE PORTA (DATA) OF 8255 AS INPUTS */
        outp(DIOBASE + DIOCMD, PORTAIN);

	/* ENABLE RS and R/W SIGNAL TO LCD */
        outp(DIOBASE + DIOPORTB, LCDDATA | LCDRD);

	/* ENABLE E SIGNAL TO LCD */
        outp(DIOBASE + DIOPORTB, LCDDATA | LCDRD | LCD_E);

	/* READ FROM THE LCD */
        tempch = inp(DIOBASE + DIOPORTA);

        /* THE FOLLOWING LINE REVERSES THE DATA BITS TO SIMPLIFY */
        /* THE INTERFACE CABLE - SEE DOCUMENTATION FOR DETAILS */
        ret_LCD = reverse_data(tempch); 

	/* DISABLE E SIGNAL TO LCD */
        outp(DIOBASE + DIOPORTB, LCDDATA );

	return (ret_LCD);
}

/***************************************************
 *      reverse_data()                             *
 *                                                 *
 *      This function will reverse the bits of a   *
 *      data byte to simplify interface cable.     *
 *      See DOCUMENTATION for cable details.       *
 ***************************************************/
unsigned char reverse_data(unsigned char ch)
{
        int n;
        unsigned char retval;
        retval = 0;

        for(n=0; n<8; n++)
        {
           if( ch & (1<<n))
              retval |= (1<<(7-n));
        }
        return (retval);
}

