/**********************************************************
 *      LCD DRIVER for 82C55 Digital I/O                  *
 *      SW4057                                            *
 *      REV 1.00                                          *
 *      5/24/99                                           *
 *                                                        *
 *      (c)1999 Micro/sys, Inc.                           *
 *      Glendale, CA                                      *
 *                                                        *
 *      This file contains the routines used to access    *
 *      an LCD display interfaced to an 82C55.            *
 **********************************************************/
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <time.h>
#include "LCD8255.h"

extern int DIOBASE;
/***************************************************************
 *      LCDClearline()                                         *
 *                                                             *
 *      This function will clear the specified line on LCD.    *
 ***************************************************************/
void LCDClearline(int line)
{
        int n;

        if(line==1 || line==2)
        {
        LCDSetCursor(1, line);
	for(n=1; n<41; n++)
                LCDPutc(' ');    /* write a space */
        }
}


/***************************************************************
 *      LCDScroll()                                            *
 *                                                             *
 *      This function will copy the characters on line 2 to    *
 *      line 1 of the LCD display and then clear line 2.       *
 ***************************************************************/
void LCDScroll(void)
{
	int n;
	unsigned char ch;

	for(n=1; n<41; n++)
	{
                LCDSetCursor(n, 2);
                wait_for_LCD();        /* wait for LCD to be ready */
		ch = read_data_LCD();  /* read char from 2nd line */
                LCDSetCursor(n, 1);    /* go to the 1st line */
                LCDPutc(ch);           /* write character to the 1st line */
	}
        LCDClearline(2);
}


/*************************************************
 *      LCDPutc()                                *
 *                                               *
 *      This function will output a character    *
 *      to the LCD in the current position.      *
 *************************************************/
int LCDPutc(int c)
{
	int Xpos, Ypos;
	unsigned char status;

	wait_for_LCD();
	status = read_control_LCD() & 0x7F;
	Xpos = (status & 0x3F)+1;
	Ypos = ((status & 0x40)>>6)+1;

	switch(c)
	{
		case '\r':      /* carriage-return */
                   LCDSetCursor(1, Ypos);
                   break;

		case '\n':
                   if (Ypos == 1)
                   {
                      LCDSetCursor(1, 2); /* go to start of 2nd line */
                      break;
                   }
                   LCDScroll();            /* copy 2nd line to 1st */
                   LCDSetCursor(1, 2);     /* go to start of 2nd line */
                   break;

		case '\b':      /* backspace */
                   if(Xpos != 1)
                      LCDSetCursor(Xpos-1, Ypos);
                   break;

		default:
                   wait_for_LCD();     /* wait for LCD to be ready */
                   write_data_LCD((unsigned char)c);  /* output char  */
                   if (Xpos == 40)
                      LCDSetCursor(40, Ypos);
	}
	return (c);     /* always return the character to show success */
}
