/**********************************************************
 *      LCD DRIVER for 82C55 Digital I/O                  *
 *      SW4057                                            *
 *      REV 1.00                                          *
 *      5/24/99                                           *
 *                                                        *
 *      (c)1999 Micro/sys, Inc.                           *
 *      Glendale, CA                                      *
 *                                                        *
 *      This file contains the routines used to access    *
 *      an LCD display interfaced to an 82C55.            *
 **********************************************************/
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <time.h>
#include "LCD8255.h"

/************************************************************
 *      int10_handler()                                     *
 *                                                          *
 *      This interrupt handler function takes the place     *
 *      of the standard Int 10h handler in order to         *
 *      redirect the standard console output to the LCD.    *
 ************************************************************/
void interrupt cdecl far int10_handler( unsigned _es, unsigned _ds,
	     unsigned _di, unsigned _si, unsigned _bp, unsigned _sp,
	     unsigned _bx, unsigned _dx, unsigned _cx, unsigned _ax,
	     unsigned _ip, unsigned _cs, unsigned _flags)
{
	unsigned char _ah, _dh, _dl;
	int status, n, Xpos, Ypos;

	_dh = (_dx & 0xFF00)>>8;
	_dl = _dx & 0xFF;
	_ah = (_ax & 0xFF00)>>8;

	switch(_ah)
	{
		case 0x2:       /* set cursor position */
                   LCDSetCursor(_dl+1, _dh+1);
                   break;

		case 0x3:       /* read cursor position and configuration */
                   wait_for_LCD();
                   status = read_control_LCD() & 0x7F;
                   Xpos = status & 0x3F;
                   Ypos = (status & 0x40)>>6;
                   _dx = (Ypos<<8)+ Xpos;
                   break;

		case 0x8:       /* read character and attribute */
                   wait_for_LCD();   /* wait for LCD to be ready */
                   _ax = (unsigned) read_data_LCD();  /* read character */
                   break;

		case 0x9:       /* write character and attribute */
                   for(n=0;n<_cx; n++);
                      LCDPutc(_ax & 0xFF);
                   break;

		case 0xE:       /* write text in teletype mode */
                   LCDPutc(_ax & 0xFF);
                   break;

		case 0xF:       /* get current display mode */
                   _ax = 0x5007;   /* 80 columns, mode 07 */
                   _bx &= 0xFF;    /* bh = 0, page 0 */
                   break;

		default:
                   break;
	}
}

/**************************************************************
 *      LCDInstallVideo()                                     *
 *                                                            *
 *      This function will install a handler for Int 10h      *
 *      (video) that will redirect output to the LCD.         *
 *                                                            *
 **************************************************************/
void LCDInstallVideo(void)
{
static void (interrupt far *OrigInt10Vect)( void );

        OrigInt10Vect = _dos_getvect (0x10);    /* save old handler */

        _dos_setvect(0x10, int10_handler);      /* install new handler */
}


/**************************************************************
 *      LCDUninstallVideo()                                   *
 *                                                            *
 *      This function will uninstall the handler for          *
 *      Int 10h (video) that redirects output to the LCD.     *
 *                                                            *
 **************************************************************/
void LCDUninstallVideo(void)
{
static void (interrupt far *OrigInt10Vect)( void );

        _dos_setvect(0x10, OrigInt10Vect); /* replace old handler */
}

