/***************************************************************
 *      PRINTER DRIVER for 82C55 Digital I/O                   *
 *      SW4057                                                 *
 *      REV 1.00                                               *
 *      6/1/99                                                 *
 *                                                             *
 *      (c)1999 Micro/sys, Inc.                                *
 *      Glendale, CA                                           *
 *                                                             *
 *      This file contains the routines used to access         *
 *      a parallel printer port interfaced to an 82C55.        *
 ***************************************************************/
#include <stdio.h>
#include <bios.h>
#include <dos.h>
#include "LPT8255.h"

extern int DIOBASE;

/************************************************************
 *    LPTInt17io()                                          *
 *                                                          *
 *    This function implements the _bios_printer services   *
 *    of INT 0x17 on the 82C55 digital I/O port after the   *
 *    redirection of INT17 interrupt with redirect_LPT().   *
 ************************************************************/
void interrupt far LPTInt17io()
{
unsigned int axreg, dxreg, n;
unsigned int timeout = 40000;
unsigned char character;
unsigned char function;
unsigned char LPTport;

    _asm mov ax, [bp+18]         /* get AX from stack */
    _asm mov axreg, ax

    _asm mov dx, [bp+14]         /* get DX from stack */
    _asm mov dxreg, dx

    character = axreg & 0x00FF;             /* get character from AL */
    function = (axreg & 0xFF00) / 256;      /* get function from AH */
    LPTport = dxreg & 0x00FF;               /* get port from DL */

    if (function == 0)          /* function 0 - transmit 1 character */  
    {

       while ( inp(DIOBASE + DIOPORTC) & BUSY ); /* check if printer busy */
       {
       axreg = 0x80;            /* set busy bit if busy */

       if (timeout-- == 0)      /* wait for timeout period */
          axreg = 0x81;         /* set timeout bit if timeout occurs */
       }

       if (timeout == 0)    
          goto retval;


       /* put character on data lines */
       outp( DIOBASE + DIOPORTA, character );  

       outp( DIOBASE + DIOPORTB, INIT );    /* assert data strobe, not init */
       for( n=0; n<5000; n++ )             /* extend strobe pulse width */
          ;
       outp( DIOBASE + DIOPORTB, STROBE | INIT);  /* de-assert data strobe */

    retval:
       _asm mov ax, axreg                  /* set up return value */
    }


    else if ( function == 1 ) /* function 1 - initialize parallel port */
    {
       outp( DIOBASE + DIOPORTB, STROBE );      /* assert init */
       for( n=0; n<50000; n++ )                 /* extend init pulse width */
          ;
       outp( DIOBASE + DIOPORTB, STROBE | INIT );         /* de-assert init */
    }


    else if ( function == 2 ) /* function 2 - get parallel port status */
    {  
       if( inp(DIOBASE + DIOPORTC) & BUSY )     /* check if port busy */
          axreg = 0x80;         /* return busy flag */
       else
          axreg = 0x81;         /* return not busy */
       _asm mov ax, axreg       /* set up return value */
    }
}
